﻿using System.Collections.Generic;
using System.IO;
using System.Text;
using X3g.Plugin;

namespace SamplePlugin
{
    class CustomExport
    {
        public void Export(GeComponent root, string filePath)
        {
            ComponentCache = new Dictionary<GeComponent, int>();
            mMeshes = new Dictionary<GeMesh, int>();

            ComponentStream = new StringBuilder();
            ComponentStream.AppendLine("<components>");

            var sb = new StringBuilder();
            sb.AppendLine("<scene>");
            WriteComponent(root, sb);
            sb.AppendLine("</scene>");

            ComponentStream.AppendLine("</components>");

            WriteMeshes(sb);

            using (var f = new StreamWriter(filePath, false, Encoding.UTF8))
            {
                f.WriteLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
                f.WriteLine("<pplx>");
                f.Write(ComponentStream.ToString());
                f.Write(sb.ToString());
                f.WriteLine("</pplx>");
                f.Flush();
            }
        }

        private void WriteComponent(GeComponent component, StringBuilder sb)
        {
            if (!string.IsNullOrEmpty(component.Name))
                sb.AppendLine("<name>" + component.Name + "</name>");

            if (component.Instances != null)
                foreach (var i in component.Instances) WriteInstance(i, sb);

            if (component.Meshes != null)
                foreach (var i in component.Meshes) WriteMeshInstance(i, sb);

            if (component.Polylines != null)
                foreach (var i in component.Polylines) WritePolylines(i, sb);
        }

        private void WriteInstance(GeComponentInstance instance, StringBuilder sb)
        {
            // component
            int id = -1;

            if (instance.Component != null)
            {
                if (!ComponentCache.TryGetValue(instance.Component, out id))
                {
                    ComponentCache[instance.Component] = id = ComponentCache.Count;
                    var csb = new StringBuilder();
                    csb.AppendLine("<component id=\"" + id + "\">");
                    WriteComponent(instance.Component, csb);
                    csb.AppendLine("</component>");
                    ComponentStream.Append(csb.ToString());
                }
            }

            // transform
            sb.AppendLine("<componentInstance componentId=\"" + id + "\">");
            double[] m = instance.Transform.Get();
            sb.Append("<transform>");
            for (int i = 0; i < 16; ++i)
                sb.Append(m[i] + ",");
            sb.AppendLine("</transform>");
            sb.AppendLine("</componentInstance>");
        }

        private void WriteMeshInstance(GeMesh pMesh, StringBuilder pStringBuilder)
        {
            int id = -1;
            if (!mMeshes.TryGetValue(pMesh, out id)) {
                mMeshes[pMesh] = id = mMeshes.Count;
            }
            pStringBuilder.AppendLine("<meshInstance id=\"" + id + "\"/>");
        }

        private void WriteMeshes(StringBuilder pStringBuilder)
        {
            pStringBuilder.AppendLine("<meshes>");
            foreach (var mesh in mMeshes) {
                WriteMesh(mesh.Key, mesh.Value, pStringBuilder);
            }
            pStringBuilder.AppendLine("</meshes>");
        }

        private void WriteMesh(GeMesh pMesh, int pId, StringBuilder pStringBuilder)
        {
            pStringBuilder.AppendLine("<mesh id=\"" + pId + "\">");
            pStringBuilder.Append("<vertices>");
            if (pMesh.Vertices != null)
                foreach (var v in pMesh.Vertices)
                    pStringBuilder.Append(v.x + ";" + v.y + ";" + v.z + ";");
            pStringBuilder.AppendLine("</vertices>");
            pStringBuilder.AppendLine("</mesh>");
        }

        private void WritePolylines(GePolylines polyLines, StringBuilder sb)
        {
            //not implemented
        }

        private Dictionary<GeComponent, int> ComponentCache;
        private Dictionary<GeMesh, int> mMeshes;
        private StringBuilder ComponentStream;
    }
}
