﻿using NGettext;
using System.Globalization;
using System.IO;

namespace SamplePlugin
{
    /// <summary>
    /// this class will translate the string to the system locale
    /// it needs the SamplePlugin.resources.dll in the correct locale folder like de, en, fr etc
    /// </summary>
    class I18N
    {
        static ICatalog mCatalog = null;

        private static void Init()
        {
            string pluginPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location) + "\\";

            // first we try with full culture like en-US or de-DE
            string name = pluginPath + CultureInfo.CurrentUICulture.Name + ".mo";
            try
            {
                using (Stream moFilestream = File.OpenRead(name))
                {
                    mCatalog = new Catalog(moFilestream, CultureInfo.CurrentCulture);
                }
            }
            catch (System.Exception)
            {
                mCatalog = null;
            }

            // If we failed the first time, our next attempt will be to open two letter locale
            // e.g.   en instead of en-US/en-GB/...   or   de instead of de-DE/de-AT/de-CH/...
            name = pluginPath + CultureInfo.CurrentUICulture.TwoLetterISOLanguageName + ".mo";
            if (mCatalog == null)
            {
                try
                {
                    using (Stream moFilestream = File.OpenRead(name))
                    {
                        mCatalog = new Catalog(moFilestream, CultureInfo.CurrentCulture);
                    }
                }
                catch (System.Exception)
                {
                    mCatalog = null;
                }
            }

            // There are only 3 reason why mCatalog would be null
            // 1st The files are not available
            // 2nd The encoding is some ancient Codepage NGettext does not support (e.g. CP1252)
            // 3rd The Process is not allowed to read the files
            if (mCatalog == null)
                mCatalog = new Catalog();
        }

        public static string _(string pId)
        {
            // We load the Catalog on demand
            if (mCatalog == null)
                Init();
            return mCatalog.GetString(pId);
        }
    }
}
