﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using X3g.Plugin;

namespace SamplePlugin.SceneExplorer
{
    /// <summary>
    /// Interaction logic for SceneExplorerView.xaml
    /// </summary>
    public partial class SceneExplorerView : UserControl
    {
        public SceneExplorerView(SceneExplorer pSceneExplorer, DocumentManager pDocument)
        {
            mDocument = pDocument;
            mSceneExplorer = pSceneExplorer;
            mTaggedItems = new List<TreeViewItem>();
            InitializeComponent();
        }

        private void updateButton_Click(object sender, RoutedEventArgs e)
        {
            blockTreeView.Items.Clear();
            foreach (string blockName in mDocument.Blocks.GetAllBlockNames())
            {
                TreeViewItem blockItem = new TreeViewItem() { Header = blockName };
                blockItem.Selected += BlockItem_Selected;
                X3g.Plugin.Block block = mDocument.Blocks.GetBlock(blockName);
                foreach (DbEntity entity in block.GetEntities())
                {
                    string header = entity.Id + " : " + entity.GetType().Name;
                    TreeViewItem entityItem = new TreeViewItem() { Header = header };
                    entityItem.Selected += EntityItem_Selected;
                    blockItem.Items.Add(entityItem);
                }
                blockTreeView.Items.Add(blockItem); 
            }
        }

        private void BlockItem_Selected(object sender, RoutedEventArgs e)
        {
            TreeViewItem item = sender as TreeViewItem;
            foreach (TreeViewItem entityItem in item.Items)
            {
                if (entityItem.IsSelected)
                    return;
            }
            mSceneExplorer.BlockSelected(item.Header.ToString());
            TagItem("");
        }

        private void EntityItem_Selected(object sender, RoutedEventArgs e)
        {
            TreeViewItem item = sender as TreeViewItem;
            string header = item.Header.ToString();
            string entityId = header.Split(' ')[0];
            mSceneExplorer.EntitySelected(entityId);

            DbBlockReference blockRef = DbEntity.FromId(entityId) as DbBlockReference;
            TagItem(blockRef != null ? blockRef.Block : "");
        }

        private void TagItem(string pHeader)
        {
            foreach (var item in mTaggedItems)
            {
                TreeViewItem treeItem = item as TreeViewItem;
                treeItem.Foreground = Brushes.Black;
            }
            mTaggedItems.Clear();

            foreach (var item in blockTreeView.Items)
            {
                TreeViewItem treeItem = item as TreeViewItem;
                if (treeItem.Header.Equals(pHeader))
                {
                    treeItem.Foreground = Brushes.Red;
                    mTaggedItems.Add(treeItem);
                    return;
                }
            }
        }

        private List<TreeViewItem> mTaggedItems;
        private SceneExplorer mSceneExplorer;
        private DocumentManager mDocument;
    }
}
